import os
import json


def convert_bytes(bytes):
    output = 0
    if 'Mi' in bytes:
        number = bytes.replace('Mi', ' ')
        output = int(number) * 1024 * 1024
    if 'Gi' in bytes:
        number = bytes.replace('Gi', ' ')
        output = int(number) * 1024 * 1024 * 1024
    return output


def manage_dependencies(dependencies, application):
    for d in dependencies:
        name = d.get('component')
        new_name = application + "-" + name.lower()
        d.update((k, new_name) for k, v in d.items() if v == name)


def generate(nodelist, application):
    json_template = {}
    resourcelist = []

    for x in nodelist:
        if x.get_type() == 'tosca.nodes.Compute.EdgeNode':
            edge_node_name = x.get_name()
            edge_cpu = x.get_num_cpu()
            edge_disk = str(convert_bytes(x.get_disk_size()))
            edge_mem = str(convert_bytes(x.get_mem_size()))
            edge_gpu_type = x.get_gpu_dedicated()
            edge_gpu_model = x.get_gpu_model()
            edge_os = x.get_os()
            edge_architecture = x.get_architecture()
            if edge_gpu_model != "None":
                json_requirements = {'type': edge_node_name, 'os': edge_os, 'arch': edge_architecture,
                                     'hardware_requirements': {'cpu': str(edge_cpu), 'ram': edge_mem, 'disk': edge_disk,
                                                               'gpu': {'brand': edge_gpu_model,
                                                                       'dedicated': str(edge_gpu_type)}}}
                resourcelist.append(json_requirements)
            else:
                json_requirements = {'type': edge_node_name, 'os': edge_os, 'arch': edge_architecture,
                                     'hardware_requirements': {'cpu': str(edge_cpu), 'ram': edge_mem, 'disk': edge_disk}
                                     }
                resourcelist.append(json_requirements)

        if x.get_type() == 'tosca.nodes.Compute.PublicCloud':
            vm_node_name = x.get_name()
            vm_cpu = x.get_num_cpu()
            vm_disk = str(convert_bytes(x.get_disk_size()))
            vm_mem = str(convert_bytes(x.get_mem_size()))
            vm_os = x.get_os()
            vm_arch = x.get_architecture()
            json_requirements = {'type': vm_node_name, 'os': vm_os, 'arch': vm_arch,
                                 'hardware_requirements': {'cpu': str(vm_cpu), 'ram': vm_mem, 'disk': vm_disk}
                                 }
            resourcelist.append(json_requirements)
    json_template[application] = []
    nodelist = [i for i in nodelist if i.get_type() != "tosca.nodes.Compute.EdgeNode"]
    nodelist = [i for i in nodelist if i.get_type() != "tosca.nodes.Compute.PublicCloud"]
    nodelist = [i for i in nodelist if i.get_type() != "ACCORDION.Cloud_Framework"]

    for x in nodelist:
        for y in resourcelist:
            if x.get_host() == y.get('type'):
                print(y)
                print(x.get_name())
                unit = x.get_unit()
                name = x.get_name()
                if x.get_port():
                    port = x.get_port()
                    if type(port) != list:
                        port_list = []
                        port_list.append(port)
                        host = x.get_host()
                        result = ''.join([i for i in host if not i.isdigit()])

                        dependecy = x.get_dependency()
                        if not dependecy:
                            json_template[application].append({
                                'component': application + "-" + name,
                                'unit': unit,
                                'port': port_list,
                                'host': {
                                    'host_type': result,
                                    'requirements': y
                                }
                            })
                        if dependecy:
                            manage_dependencies(dependecy, application)
                            json_template[application].append({
                                'component': application + "-" + name,
                                'unit': unit,
                                'port': port_list,
                                'dependency': dependecy,
                                'host': {
                                    'host_type': result,
                                    'requirements': y
                                }
                            })
                    else:
                        host = x.get_host()
                        result = ''.join([i for i in host if not i.isdigit()])
                        dependecy = x.get_dependency()
                        if not dependecy:
                            json_template[application].append({
                                'component': application + "-" + name,
                                'unit': unit,
                                'port': port,
                                'host': {
                                    'host_type': result,
                                    'requirements': y
                                }
                            })
                        if dependecy:
                            manage_dependencies(dependecy, application)
                            json_template[application].append({
                                'component': application + "-" + name,
                                'unit': unit,
                                'port': port,
                                'dependency': dependecy,
                                'host': {
                                    'host_type': result,
                                    'requirements': y
                                }
                            })
                else:
                    host = x.get_host()
                    result = ''.join([i for i in host if not i.isdigit()])
                    dependecy = x.get_dependency()
                    if not dependecy:
                        json_template[application].append({
                            'component': application + "-" + name,
                            'unit': unit,
                            'host': {
                                'host_type': result,
                                'requirements': y
                            }
                        })
                    if dependecy:
                        manage_dependencies(dependecy, application)
                        json_template[application].append({
                            'component': application + "-" + name,
                            'unit': unit,
                            'dependency': dependecy,
                            'host': {
                                'host_type': result,
                                'requirements': y
                            }
                        })
    return json_template



class CloudFramework:
    def set_type(self, type):
        self.__type = type

    def get_type(self):
        return self.__type

    def set_secret_name(self, secret_name):
        self.__secret_name = secret_name

    def get_secret_name(self):
        return self.__secret_name

    def set_application(self, app):
        self._app = app

    def get_application(self):
        return self._app

    def set_literals(self, literals):
        self.__literals = literals

    def get_literals(self):
        return self.__literals

    def set_resources(self, resources):
        self.__resources = resources

    def get_resources(self):
        return self.__resources

    def set_order(self, order):
        self.__order = order

    def get_order(self):
        return self.__order

    def set_target(self, target):
        self.__target = target

    def set_relationship(self, relationship):
        self.__relationship = relationship

    def get_relationship(self):
        return self.__relationship

    def set_actions(self, actiolist):
        self.__actionlist = actiolist

    def get_actions(self):
        return self.__actionlist

    def set_workflows(self, workflowlist):
        self.__workflowlist = workflowlist

    def get_workflows(self):
        return self.__workflowlist



import random
import string
import time


def generate_k3s_pod_name(appinstanceinfo, component_name, minicloud_id):
    running_component_id = ''.join(random.choices(string.ascii_lowercase + string.digits, k=5))
    k3s_pod_name = appinstanceinfo + "-" + component_name + "-" + str(
        running_component_id) + "-" + minicloud_id
    return k3s_pod_name


def generate_k3s_namespace(application_name, application_version, application_id):
    k3s_namespace = "acc-" + application_name + "-" + application_version + "-" + application_id
    return k3s_namespace



import os
from converter_package import Action

from converter_package import Repository
from converter_package import ComputeNode

from converter_package import CloudFramework
from converter_package import Container
from converter_package import Image
from converter_package import Workflows

home = str(os.getcwd())


def ReadFile(json):
    nodelist = []

    imagelist = []
    application = ""
    requirements = json.get('requirements')
    definitions = requirements[0].get('toscaDescription')
    topology = definitions.get('topology_template')
    node_template = topology.get('node_templates')
    registry = json.get('registry')
    application_details = json.get('details')
    application_version = application_details.get('version')
    print(registry)
    repolist = []
    for repository in registry:
        repo = Repository.Repository()
        repo.set_version(repository.get('version'))
        repo.set_imageName(repository.get('imageName'))
        repo.set_path(repository.get('location'))
        print(repository)
        repo.set_component(repository.get('component'))
        repolist.append(repo)
    node_names = node_template.keys()
    print(node_names)
    for x in node_names:
        node = node_template.get(x)
        type = node.get('type')
        properties = node.get('properties')
        if 'Cloud_Framework' in type:
            cloud = CloudFramework.CloudFramework()
            cloud.set_type(type)
            application = properties.get('application')
            cloud.set_application(application)
            if 'deployment_phase' in properties:
                actions = properties.get('deployment_phase')
                actionlist = []
                for actionset in actions:
                    component_names = []
                    action = Action.Action()
                    action.set_name(actionset.get('name'))
                    action.set_order(actionset.get('order'))
                    images = actionset.get('components')
                    for image in images:
                        print(image)
                        object = Image.Image()
                        object.set_image_type(image.get('type'))
                        component_names.append(image.get('component').lower())
                        object.set_path(image.get('component').lower())
                        object.set_component(image.get('component'))
                        if not imagelist:
                            imagelist.append(object)
                        if imagelist[-1].get_path() != image.get('component').lower():
                            imagelist.append(object)

                    actionlist.append(
                        {'action': action.get_name(), 'order': action.get_order(),
                         'components': images})
                cloud.set_actions(actionlist)
            nodelist.append(cloud)
            if 'workflows' in properties:
                workflows = properties.get('workflows')
                workflowlist = []
                actionlist = []
                for workflowset in workflows:
                    workflow = Workflows.Workflows()
                    workflow.set_scenario(workflowset.get('scenario'))
                    workflow.set_condition(workflowset.get('condition'))
                    actions = workflowset.get('actions')
                    actionlist = []
                    for actionset in actions:
                        component_names = []
                        action = Action.Action()
                        action.set_name(actionset.get('name'))
                        action.set_order(actionset.get('order'))
                        if action.get_name() != 'send':
                            images = actionset.get('components')
                            for image in images:
                                print(image)
                                object = Image.Image()
                                object.set_image_type(image.get('type'))
                                component_names.append(image.get('component').lower())
                                object.set_path(image.get('component').lower())
                                object.set_component(image.get('component'))

                            actionlist.append(
                                {'action': action.get_name(), 'order': action.get_order(),
                                 'components': images})
                        if action.get_name() == 'send':
                            if actionset.get('input'):
                                target_list = actionset.get('input')
                            for target in target_list:
                                target.update(component=target.get('component').lower(), to=target.get('to').lower())

                            actionlist.append(
                                {'action': action.get_name(), 'order': action.get_order(),
                                 'send': target_list})
                    workflowlist.append({'scenario': workflow.get_scenario(), 'condition': workflow.get_condition(),
                                         'actions': actionlist})
                cloud.set_workflows(workflowlist)
        if 'Component' in type:
            container = Container.Container()
            container.set_type(type)
            name = properties.get('name')
            container.set_name(name)
            container.set_application(application)
            if properties.get('instance'):
                instance_num = properties.get('instance')
                container.set_instance(instance_num)
            else:
                container.set_instance(1)
            service = properties.get('external_ip')
            container.set_service(service)
            unit = properties.get('deployment_unit')
            if unit == "vm":
                flavor = properties.get("flavor")
                container.set_flavor(flavor)
            container.set_unit(unit)
            ports = properties.get('ports')
            if ports:
                container.set_port(ports)
            else:
                container.set_port(None)
            storage_type = properties.get('storage_type')
            container.set_storage_type(storage_type)
            if storage_type == "persistent":
                container.set_volumeMounts_name(name + '-persistent-storage')
                container.set_volumeMounts_path('/var/lib/' + name)
                container.set_volumes_name(name + '-persistent-storage')
                container.set_volumes_claimname(name + '-pv-claim')
            if properties.get('dependency'):
                dependency = properties.get('dependency')
                container.set_dependency(dependency)
            else:
                container.set_dependency(None)
            requirements = node.get('requirements')[0]
            host = requirements.get('host')
            container.set_host(host)
            # here is done the matching between components of the application model and images of the intermediate model
            # VM part is not fully developed since we don't have the description of these images at IM
            for image in imagelist:
                if name == image.get_path():
                    if image.get_image_type() == 'vm':
                        container.set_image(image.get_component().lower())
                    for repo in repolist:
                        if repo.get_component() == image.get_path():
                            container.set_image(repo.get_path())
            nodelist.append(container)
        if 'EdgeNode' in type:
            edgenode = ComputeNode.ComputeNode()
            edgenode.set_name(x)
            edgenode.set_type(type)
            if properties:
                if properties.get('gpu_model'):
                    gpu_model = properties.get('gpu_model')
                    gpu_properties = gpu_model.get('properties')
                    if gpu_properties.get('model'):
                        model = gpu_properties.get('model')
                        edgenode.set_gpu_model(model)
                    if gpu_properties.get('brand'):
                        brand = gpu_properties.get('brand')
                        edgenode.set_gpu_brand(brand)
                    if not gpu_properties.get('brand'):
                        edgenode.set_gpu_brand("None")
                    dedicated = gpu_properties.get('dedicated')
                    edgenode.set_gpu_dedicated(dedicated)
                wifi_antenna = properties.get('wifi_antenna')
                if wifi_antenna:
                    edgenode.set_wifi_antenna(wifi_antenna)
                if not wifi_antenna:
                    edgenode.set_wifi_antenna("None")
                if not properties.get('gpu_model'):
                    edgenode.set_gpu_model("None")
                    edgenode.set_gpu_dedicated("None")
                    edgenode.set_gpu_brand("None")
            if not properties:
                edgenode.set_gpu_model("None")
                edgenode.set_gpu_dedicated("None")
                edgenode.set_wifi_antenna("None")
                edgenode.set_gpu_brand("None")
            capabilities = node.get('capabilities')
            if capabilities.get('host'):
                host = capabilities.get('host')
                host_properties = host.get('properties')
                if host_properties.get('num_cpus'):
                    num_cpus = host_properties.get('num_cpus')
                    edgenode.set_num_cpu(num_cpus)
                else:
                    edgenode.set_num_cpu(None)
                mem_size = host_properties.get('mem_size')
                edgenode.set_mem_size(mem_size)
                if host_properties.get('disk_size') is not None:
                    disk_size = host_properties.get('disk_size')
                    edgenode.set_disk_size(disk_size)
                else:
                    edgenode.set_disk_size("200 MB")
            else:
                edgenode.set_num_cpu(None)
                edgenode.set_mem_size(None)
                edgenode.set_disk_size("200 MB")
            os = capabilities.get('os')
            os_properties = os.get('properties')
            os_type = os_properties.get('type')
            architecture = os_properties.get('architecture')
            edgenode.set_architecture(architecture)
            edgenode.set_os(os_type)
            nodelist.append(edgenode)
        if 'PublicCloud' in type:
            vm = ComputeNode.ComputeNode()
            vm.set_name(x)
            vm.set_type(type)
            capabilities = node.get('capabilities')
            if capabilities.get('host'):
                host = capabilities.get('host')
                host_properties = host.get('properties')
                num_cpus = host_properties.get('num_cpus')
                vm.set_num_cpu(num_cpus)
                mem_size = host_properties.get('mem_size')
                vm.set_mem_size(mem_size)
                disk_size = host_properties.get('disk_size')
                vm.set_disk_size(disk_size)
            else:
                vm.set_num_cpu(None)
                vm.set_mem_size(None)
                vm.set_disk_size("200 MB")
            os = capabilities.get('os')
            os_properties = os.get('properties')
            os_type = os_properties.get('type')
            vm.set_os(os_type)
            architecture = os_properties.get('architecture')
            vm.set_architecture(architecture)
            nodelist.append(vm)

    return (nodelist), (imagelist), application_version.replace(".", "-")



from hurry.filesize import size, iec


def convert_bytes(bytes):
    output = 0
    if 'MB' in bytes:
        number = bytes.replace('MB', ' ')
        output = int(number) * 1024 * 1024
    if 'GB' in bytes:
        number = bytes.replace('GB', ' ')
        output = int(number) * 1024 * 1024 * 1024
    return output


class ComputeNode:
    def set_name(self, name):
        self.__name = name

    def get_name(self):
        return self.__name
    def set_type(self, type):
        self.__type = type

    def get_type(self):
        return self.__type

    def set_disk_size(self, disk_size):
        disk_size = convert_bytes(disk_size)
        self.__disk_size = size(disk_size, system=iec)

    def get_disk_size(self):
        return self.__disk_size

    def set_num_cpu(self, cpu):
        self.__cpu = cpu

    def get_num_cpu(self):
        return self.__cpu

    def set_mem_size(self, mem_size):
        if mem_size is not None:
            mem_size = convert_bytes(mem_size)
            self.__mem_size = size(mem_size, system=iec)
        if mem_size is None:
            self.__mem_size = mem_size

    def get_mem_size(self):
        return self.__mem_size

    def set_gpu_model(self, model):
        self.__model = model

    def get_gpu_model(self):
        return self.__model

    def set_gpu_brand(self, brand):
        self.__brand = brand

    def get_gpu_brand(self):
        return self.__brand

    def set_gpu_dedicated(self, dedicated):
        self.__dedicated = dedicated

    def get_gpu_dedicated(self):
        return self.__dedicated

    def set_ip(self, ip):
        self.__ip = ip

    def get_ip(self):
        return self.__ip

    def set_relationship(self, relationship):
        self.__relationship = relationship

    def get_relationship(self):
        return self.__relationship

    def set_os(self, os):
        self.__os = os

    def get_os(self):
        return self.__os

    def set_architecture(self, arch):
        self.__arch = arch

    def get_architecture(self):
        return self.__arch

    def set_wifi_antenna(self, wifi):
        self.__wifi_antenna = wifi

    def get_wifi_antenna(self):
        return self.__wifi_antenna


class Resource:
    def set_disk(self, disk_size):
        self.__disk_size = disk_size

    def get_disk(self):
        return self.__disk_size

    def set_cpu(self, cpu):
        self.__cpu = cpu

    def get_cpu(self):
        return self.__cpu

    def set_mem(self, mem_size):
        self.__mem_size = mem_size

    def get_mem(self):
        return self.__mem_size

    def set_os(self, os):
        self.__os = os

    def get_os(self):
        return self.__os

    def set_name(self, name):
        self.__name = name

    def get_name(self):
        return self.__name

    def set_gpu_brand(self, brand):
        self.__brand = brand

    def get_gpu_brand(self):
        return self.__brand

    def set_gpu_model(self, model):
        self.__model = model

    def get_gpu_model(self):
        return self.__model

    def set_gpu_dedicated(self, dedicated):
        self.__dedicated = dedicated

    def get_gpu_dedicated(self):
        return self.__dedicated

    def set_wifi_antenna(self, wifi):
        self.__wifi_antenna = wifi

    def get_wifi_antenna(self):
        return self.__wifi_antenna

    def set_arch(self, arch):
        self.__arch = arch

    def get_arch(self):
        return self.__arch





from setuptools import setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
setup(name='converter_package',
      version='3.0',
      description='This converter library is able to transform the ACCORDION application model to K3s configuration files',
      author='Giannis Korontanis',
      author_email='gkorod2@gmail.com',
      license='MIT',
      packages=['converter_package'],
      long_description=long_description,
      long_description_content_type='text/markdown',
      install_requires=[
          'pyyaml==5.3.1', 'kafka==1.3.5', 'hurry.filesize==0.9', 'oyaml==1.0'
      ],
      zip_safe=False)



class Container:
    def set_relatioship(self, host):
        self.__host = host

    def get_relationship(self):
        return self.__host

    def set_type(self, type):
        self.__type = type

    def get_type(self):
        return self.__type

    def set_host(self, host):
        self.__host = host

    def get_host(self):
        return self.__host

    def set_port(self, port):
        self.__port = port

    def get_port(self):
        return self.__port

    def set_name(self, name):
        self.__name = name

    def get_name(self):
        return self.__name

    def set_tier(self, tier):
        self.__tier = tier

    def get_tier(self):
        return self.__tier

    def set_image_pull_policy(self, policy):
        self.__policy = policy

    def get_image_pull_policy(self):
        return self.__policy

    def set_username(self, username):
        self.__username = username

    def get_username(self):
        return self.__username

    def set_uservalue(self, uservalue):
        self.__uservalue = uservalue

    def get_uservalue(self):
        return self.__uservalue

    def set_passwordname(self, passwordname):
        self.__passwordname = passwordname

    def get_passwordname(self):
        return self.__passwordvalue

    def set_secretname(self, secretname):
        self.__secretname = secretname

    def get_secretname(self):
        return self.__secretname

    def set_secretkey(self, secretkey):
        self.__secretkey = secretkey

    def get_secretkey(self):
        return self.__secretkey

    def set_volumeMounts_name(self, volumemounts_name):
        self.__volumemounts_name = volumemounts_name

    def get_volumeMounts_name(self):
        return self.__volumemounts_name

    def set_volumeMounts_path(self, volumemounts_path):
        self.__volumemounts_path = volumemounts_path

    def get_volumeMounts_path(self):
        return self.__volumemounts_path

    def set_volumes_name(self, volumes_name):
        self.__volumes_name = volumes_name

    def get_volumes_name(self):
        return self.__volumes_name

    def set_volumes_claimname(self, volumes_claimname):
        self.__volumes_claimname = volumes_claimname

    def get_volumes_claimname(self):
        return self.__volumes_claimname

    def set_env(self, env):
        self.__env = env

    def get_env(self):
        return self.__env

    def set_namespace(self, namespace):
        self.__namespace = namespace

    def get_namespace(self):
        return self.__namespace

    def set_application(self, app):
        self.__app = app

    def get_application(self):
        return self.__app

    def set_service(self, service):
        self.__service = service

    def get_service(self):
        return self.__service

    def set_ingress(self, ingress):
        self.__ingress = ingress

    def get_ingress(self):
        return self.__ingress

    def set_image(self, images):
        self.__images = images

    def get_image(self):
        return self.__images

    def set_internal(self, boolean):
        self.__internal = boolean

    def get_internal(self):
        return self.__internal

    def set_unit(self, unit):
        self.__unit = unit

    def get_unit(self):
        return self.__unit

    def set_flavor(self, flavor):
        self.__flavor = flavor

    def get_flavor(self):
        return self.__flavor

    def set_dependency(self, dependency):
        self.__dependency = dependency

    def get_dependency(self):
        return self.__dependency

    def set_storage_type(self,storage_type):
        self.__storage_type = storage_type

    def get_storage_type(self):
        return self.__storage_type

    def set_instance(self,instance):
        self.__instance = instance

    def get_instance(self):
        return self.__instance



class Workflows:
    def set_actionlist(self, actionlist):
        self.__actionlist = actionlist

    def get_actionlist(self):
        return self.__actionlist

    def set_scenario(self, scenario):
        self.__scenario = scenario

    def get_scenario(self):
        return self.__scenario

    def set_condition(self, condition):
        self.__condition = condition

    def get_condition(self):
        return self.__condition





import json
import logging
from json import dumps, loads

from kafka import KafkaProducer, KafkaConsumer

logging.basicConfig(level=logging.INFO)


class Producer:
    def send_message(self, topic, data, ip, port):


        producer = KafkaProducer(bootstrap_servers=ip+":"+port,
                                     compression_type='gzip',
                                     max_request_size=3173440261,
                                     value_serializer=lambda x:
                                     dumps(x, ensure_ascii=False).encode('utf-8'))
        producer.send(topic, value=data)
        producer.flush()



class Image:

    def set_component(self, component):
        self.__component = component

    def get_component(self):
        return self.__component

    def set_image_name(self, image):
        self.__image = image

    def get_image_name(self):
        return self.__image

    def set_image_type(self, type):
        self.__type = type

    def get_image_type(self):
        return self.__type

    def set_path(self, path):
        self.__path = path

    def get_path(self):
        return self.__path






import oyaml as yaml
from converter_package import ID
from converter_package import ComputeNode
from converter_package import Parser
import re


def undeploy(pod, intermediate_model):
    application_instance, component_name = find_application_instance(pod)
    label = application_instance + "-" + component_name
    nodelist, imagelist, application_version = Parser.ReadFile(intermediate_model)
    deployment = {}
    vm_flag = False
    print(application_instance)
    resource_list = []
    for x in nodelist:
        if ('EdgeNode' in x.get_type()) or ('PublicCloud' in x.get_type()):
            resource = ComputeNode.Resource()
            resource.set_os(x.get_os())
            resource.set_cpu(x.get_num_cpu())
            resource.set_mem(x.get_mem_size())
            resource.set_disk(x.get_disk_size())
            resource.set_name(x.get_name())
            if x.get_architecture() == 'x86_64':
                resource.set_arch('amd64')
            else:
                resource.set_arch(x.get_architecture())
            if 'EdgeNode' in x.get_type():
                resource.set_gpu_brand(x.get_gpu_brand())
                resource.set_gpu_dedicated(x.get_gpu_dedicated())
                resource.set_wifi_antenna(x.get_wifi_antenna())
            else:
                resource.set_gpu_brand("None")
                resource.set_gpu_dedicated("None")
                resource.set_wifi_antenna("None")
            resource_list.append(resource)
        if 'Component' in x.get_type():
            if x.get_name() in pod:
                port_yaml = []
                if x.get_unit() == 'container':
                    host = x.get_host()
                    for resource in resource_list:
                        if resource.get_cpu() and resource.get_mem() and resource.get_disk():
                            resource_yaml = {
                                'requests': {'cpu': resource.get_cpu(),
                                             'memory': resource.get_mem(),
                                             'ephemeral-storage': resource.get_disk()}}
                        if resource.get_mem() and resource.get_disk() and not resource.get_cpu():
                            resource_yaml = {
                                'requests': {
                                    'memory': resource.get_mem(),
                                    'ephemeral-storage': resource.get_disk()}}
                        if host == resource.get_name():
                            filelist = []
                            ports = x.get_port()
                            if ports:
                                i = 0
                                for port in ports:
                                    i = i + 1
                                    content = {'containerPort': int(port.get('port')), 'name': x.get_name() + str(i)}
                                    port_yaml.append(content)

                            if x.get_storage_type() == "none":

                                if port_yaml:
                                    deployment = {
                                        pod: {'apiVersion': 'apps/v1',
                                              'kind': 'Deployment',
                                              'metadata': {
                                                  'name': pod,
                                                  'namespace': application_instance,
                                                  'labels': {
                                                      'app': application_instance,
                                                      'component': pod,
                                                      'type': label}},
                                              'spec': {
                                                  'selector': {
                                                      'matchLabels': {
                                                          'app': application_instance,
                                                          'component': pod,
                                                          'type': label}},
                                                  'strategy': {
                                                      'type': 'Recreate'},
                                                  'template': {
                                                      'metadata': {
                                                          'labels': {
                                                              'app': application_instance,
                                                              'component': pod,
                                                              'type': label}},
                                                      'spec': {
                                                          'containers': [
                                                              {
                                                                  'image': x.get_image(),
                                                                  'env': [
                                                                      {
                                                                          'name': 'ACCORDION_ID',
                                                                          'value': application_instance}],
                                                                  'name': x.get_name(),
                                                                  'resources': resource_yaml,
                                                                  'imagePullPolicy': 'Always',
                                                                  'ports':
                                                                      port_yaml}],
                                                          'nodeSelector': {
                                                              'beta.kubernetes.io/os': resource.get_os(),
                                                              'beta.kubernetes.io/arch': resource.get_arch(),
                                                              'resource': ''.join(
                                                                  [i for
                                                                   i
                                                                   in
                                                                   resource.get_name()
                                                                   if
                                                                   not i.isdigit()])},
                                                          'imagePullSecrets': [
                                                              {
                                                                  'name': application_instance + '-registry-credentials'}]}}}}}
                                    deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                              resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                                if not port_yaml:
                                    deployment = {
                                        pod: {'apiVersion': 'apps/v1',
                                              'kind': 'Deployment',
                                              'metadata': {
                                                  'name': pod,
                                                  'namespace': application_instance,
                                                  'labels': {
                                                      'app': application_instance,
                                                      'component': pod,
                                                      'type': label}},
                                              'spec': {
                                                  'selector': {
                                                      'matchLabels': {
                                                          'app': application_instance,
                                                          'component': pod,
                                                          'type': label}},
                                                  'strategy': {
                                                      'type': 'Recreate'},
                                                  'template': {
                                                      'metadata': {
                                                          'labels': {
                                                              'app': application_instance,
                                                              'component': pod,
                                                              'type': label}},
                                                      'spec': {
                                                          'containers': [
                                                              {
                                                                  'image': x.get_image(),
                                                                  'env': [
                                                                      {
                                                                          'name': 'ACCORDION_ID',
                                                                          'value': application_instance}],
                                                                  'name': x.get_name(),
                                                                  'resources': resource_yaml,
                                                                  'imagePullPolicy': 'Always',
                                                              }],
                                                          'nodeSelector': {
                                                              'beta.kubernetes.io/os': resource.get_os(),
                                                              'beta.kubernetes.io/arch': resource.get_arch(),
                                                              'resource': ''.join(
                                                                  [i for
                                                                   i
                                                                   in
                                                                   resource.get_name()
                                                                   if
                                                                   not i.isdigit()])},
                                                          'imagePullSecrets': [
                                                              {
                                                                  'name': application_instance + '-registry-credentials'}]}}}}}
                                    deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                              resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                            if x.get_storage_type() == "ephemeral":
                                if port_yaml:
                                    deployment = {
                                        pod
                                        : {'apiVersion': 'apps/v1',
                                           'kind': 'Deployment',
                                           'metadata': {
                                               'name': pod,
                                               'namespace': application_instance,
                                               'labels': {
                                                   'app': application_instance,
                                                   'component': pod,
                                                   'type': label}},
                                           'spec': {
                                               'selector': {
                                                   'matchLabels': {
                                                       'app': application_instance,
                                                       'component': pod,
                                                       'type': label}},
                                               'strategy': {
                                                   'type': 'Recreate'},
                                               'template': {
                                                   'metadata': {
                                                       'labels': {
                                                           'app': application_instance,
                                                           'component': pod,
                                                           'type': label}},
                                                   'spec': {
                                                       'containers': [
                                                           {
                                                               'image': x.get_image(),
                                                               'env': [
                                                                   {
                                                                       'name': 'ACCORDION_ID',
                                                                       'value': application_instance}],
                                                               'name': x.get_name(),
                                                               'resources': resource_yaml,
                                                               'imagePullPolicy': 'Always',
                                                               'ports':
                                                                   port_yaml,
                                                               'volumeMounts': [
                                                                   {
                                                                       'name': "cache-volume",
                                                                       'mountPath': "/cache"}]}],
                                                       'volumes': [{
                                                           'name': "cache-volume",
                                                           'emptyDir': {}}],
                                                       'nodeSelector': {
                                                           'beta.kubernetes.io/os': resource.get_os(),
                                                           'beta.kubernetes.io/arch': resource.get_arch(),
                                                           'resource': ''.join(
                                                               [i for
                                                                i
                                                                in
                                                                resource.get_name()
                                                                if
                                                                not i.isdigit()])},
                                                       'imagePullSecrets': [
                                                           {
                                                               'name': application_instance + '-registry-credentials'}]}}}}}
                                    deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                              resource.get_gpu_dedicated(),
                                                              resource.get_wifi_antenna())

    return deployment


def find_replica_instance(componentInfo):
    # removes minID
    namespace = re.sub(r'min[\d+$]', "", componentInfo)
    # stored minID
    minID = componentInfo.replace(namespace, "")
    # retrieve component name
    namespace = namespace[:-7]
    reverse_namespace = namespace[::-1]
    reverse_component_name = reverse_namespace.split("-", 1)[0]
    actual_component_name = reverse_component_name[::-1]
    label = namespace
    namespace = namespace.replace("-" + actual_component_name, "")
    pod_name = ID.generate_k3s_pod_name(namespace, actual_component_name, minID)
    return namespace, pod_name, label


def find_application_instance(componentInfo):
    # removes minID
    namespace = re.sub(r'min[\d+$]', "", componentInfo)
    # removes runningComponentID
    namespace = namespace[:-7]
    letters = []
    # removes actual component name
    for character in namespace:
        m = re.search(r'\d+$', namespace)
        if m is None:
            letters.append(namespace[len(namespace) - 1])
            namespace = namespace[:-1]
    letters.reverse()
    letters.pop(0)
    component_name = ""
    for letter in letters:
        component_name = component_name + letter
    return namespace, component_name


def scale_out_to_k8s(pod, intermediate_model):
    application_instance, component_name, label = find_replica_instance(pod)
    nodelist, imagelist, application_version = Parser.ReadFile(intermediate_model)
    deployment = {}
    vm_flag = False
    print(application_instance)
    resource_list = []
    for x in nodelist:
        if ('EdgeNode' in x.get_type()) or ('PublicCloud' in x.get_type()):
            resource = ComputeNode.Resource()
            resource.set_os(x.get_os())
            resource.set_cpu(x.get_num_cpu())
            resource.set_mem(x.get_mem_size())
            resource.set_disk(x.get_disk_size())
            resource.set_name(x.get_name())
            if x.get_architecture() == 'x86_64':
                resource.set_arch('amd64')
            else:
                resource.set_arch(x.get_architecture())
            if 'EdgeNode' in x.get_type():
                resource.set_gpu_brand(x.get_gpu_brand())
                resource.set_gpu_dedicated(x.get_gpu_dedicated())
                resource.set_wifi_antenna(x.get_wifi_antenna())
            else:
                resource.set_gpu_brand("None")
                resource.set_gpu_dedicated("None")
                resource.set_wifi_antenna("None")
            resource_list.append(resource)
        if 'Component' in x.get_type():
            if x.get_name() in pod:
                port_yaml = []
                if x.get_unit() == 'container':
                    host = x.get_host()
                    for resource in resource_list:
                        if resource.get_cpu() and resource.get_mem() and resource.get_disk():
                            resource_yaml = {
                                'requests': {'cpu': resource.get_cpu(),
                                             'memory': resource.get_mem(),
                                             'ephemeral-storage': resource.get_disk()}}
                        if resource.get_mem() and resource.get_disk() and not resource.get_cpu():
                            resource_yaml = {
                                'requests': {
                                    'memory': resource.get_mem(),
                                    'ephemeral-storage': resource.get_disk()}}
                        if host == resource.get_name():
                            filelist = []
                            ports = x.get_port()
                            if ports:
                                i = 0
                                for port in ports:
                                    i = i + 1
                                    content = {'containerPort': int(port.get('port')), 'name': x.get_name() + str(i)}
                                    port_yaml.append(content)

                            if x.get_storage_type() == "none":

                                if port_yaml:
                                    deployment = {
                                        component_name: {'apiVersion': 'apps/v1',
                                                         'kind': 'Deployment',
                                                         'metadata': {
                                                             'name': component_name,
                                                             'namespace': application_instance,
                                                             'labels': {
                                                                 'app': application_instance,
                                                                 'component': component_name,
                                                                 'type': label}},
                                                         'spec': {
                                                             'selector': {
                                                                 'matchLabels': {
                                                                     'app': application_instance,
                                                                     'component': component_name,
                                                                     'type': label}},
                                                             'strategy': {
                                                                 'type': 'Recreate'},
                                                             'template': {
                                                                 'metadata': {
                                                                     'labels': {
                                                                         'app': application_instance,
                                                                         'component': component_name,
                                                                         'type': label}},
                                                                 'spec': {
                                                                     'containers': [
                                                                         {
                                                                             'image': x.get_image(),
                                                                             'env': [
                                                                                 {
                                                                                     'name': 'ACCORDION_ID',
                                                                                     'value': application_instance}],
                                                                             'name': x.get_name(),
                                                                             'resources': resource_yaml,
                                                                             'imagePullPolicy': 'Always',
                                                                             'ports':
                                                                                 port_yaml}],
                                                                     'nodeSelector': {
                                                                         'beta.kubernetes.io/os': resource.get_os(),
                                                                         'beta.kubernetes.io/arch': resource.get_arch(),
                                                                         'resource': ''.join(
                                                                             [i for
                                                                              i
                                                                              in
                                                                              resource.get_name()
                                                                              if
                                                                              not i.isdigit()])},
                                                                     'imagePullSecrets': [
                                                                         {
                                                                             'name': application_instance + '-registry-credentials'}]}}}}}
                                    deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                              resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                                if not port_yaml:
                                    deployment = {
                                        component_name: {'apiVersion': 'apps/v1',
                                                         'kind': 'Deployment',
                                                         'metadata': {
                                                             'name': component_name,
                                                             'namespace': application_instance,
                                                             'labels': {
                                                                 'app': application_instance,
                                                                 'component': component_name,
                                                                 'type': label}},
                                                         'spec': {
                                                             'selector': {
                                                                 'matchLabels': {
                                                                     'app': application_instance,
                                                                     'component': component_name,
                                                                     'type': label}},
                                                             'strategy': {
                                                                 'type': 'Recreate'},
                                                             'template': {
                                                                 'metadata': {
                                                                     'labels': {
                                                                         'app': application_instance,
                                                                         'component': component_name,
                                                                         'type': label}},
                                                                 'spec': {
                                                                     'containers': [
                                                                         {
                                                                             'image': x.get_image(),
                                                                             'env': [
                                                                                 {
                                                                                     'name': 'ACCORDION_ID',
                                                                                     'value': application_instance}],
                                                                             'name': x.get_name(),
                                                                             'resources': resource_yaml,
                                                                             'imagePullPolicy': 'Always',
                                                                         }],
                                                                     'nodeSelector': {
                                                                         'beta.kubernetes.io/os': resource.get_os(),
                                                                         'beta.kubernetes.io/arch': resource.get_arch(),
                                                                         'resource': ''.join(
                                                                             [i for
                                                                              i
                                                                              in
                                                                              resource.get_name()
                                                                              if
                                                                              not i.isdigit()])},
                                                                     'imagePullSecrets': [
                                                                         {
                                                                             'name': application_instance + '-registry-credentials'}]}}}}}
                                    deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                              resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                            if x.get_storage_type() == "ephemeral":
                                if port_yaml:
                                    deployment = {
                                        component_name
                                        : {'apiVersion': 'apps/v1',
                                           'kind': 'Deployment',
                                           'metadata': {
                                               'name': component_name,
                                               'namespace': application_instance,
                                               'labels': {
                                                   'app': application_instance,
                                                   'component': component_name,
                                                   'type': label}},
                                           'spec': {
                                               'selector': {
                                                   'matchLabels': {
                                                       'app': application_instance,
                                                       'component': component_name,
                                                       'type': label}},
                                               'strategy': {
                                                   'type': 'Recreate'},
                                               'template': {
                                                   'metadata': {
                                                       'labels': {
                                                           'app': application_instance,
                                                           'component': component_name,
                                                           'type': label}},
                                                   'spec': {
                                                       'containers': [
                                                           {
                                                               'image': x.get_image(),
                                                               'env': [
                                                                   {
                                                                       'name': 'ACCORDION_ID',
                                                                       'value': application_instance}],
                                                               'name': x.get_name(),
                                                               'resources': resource_yaml,
                                                               'imagePullPolicy': 'Always',
                                                               'ports':
                                                                   port_yaml,
                                                               'volumeMounts': [
                                                                   {
                                                                       'name': "cache-volume",
                                                                       'mountPath': "/cache"}]}],
                                                       'volumes': [{
                                                           'name': "cache-volume",
                                                           'emptyDir': {}}],
                                                       'nodeSelector': {
                                                           'beta.kubernetes.io/os': resource.get_os(),
                                                           'beta.kubernetes.io/arch': resource.get_arch(),
                                                           'resource': ''.join(
                                                               [i for
                                                                i
                                                                in
                                                                resource.get_name()
                                                                if
                                                                not i.isdigit()])},
                                                       'imagePullSecrets': [
                                                           {
                                                               'name': application_instance + '-registry-credentials'}]}}}}}
                                    deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                              resource.get_gpu_dedicated(),
                                                              resource.get_wifi_antenna())

    return deployment


def tosca_to_k8s(nodelist, imagelist, application_instance, minicloud, externalIP, gpu_list=[]):
    if not gpu_list:
        gpu_list = None
    deployment = {}

    service_files = []
    persistent_files = []
    deployment_files = []
    vm_flag = False
    print(application_instance)
    resource_list = []
    for x in nodelist:
        if ('EdgeNode' in x.get_type()) or ('PublicCloud' in x.get_type()):
            resource = ComputeNode.Resource()
            resource.set_os(x.get_os())
            resource.set_cpu(x.get_num_cpu())
            resource.set_mem(x.get_mem_size())
            resource.set_disk(x.get_disk_size())
            resource.set_name(x.get_name())
            if x.get_architecture() == 'x86_64':
                resource.set_arch('amd64')
            else:
                resource.set_arch(x.get_architecture())
            if 'EdgeNode' in x.get_type():
                resource.set_gpu_brand(x.get_gpu_brand())
                resource.set_gpu_dedicated(x.get_gpu_dedicated())
                resource.set_wifi_antenna(x.get_wifi_antenna())
            else:
                resource.set_gpu_brand("None")
                resource.set_gpu_dedicated("None")
                resource.set_wifi_antenna("None")
            resource_list.append(resource)
        if 'Component' in x.get_type():
            label = application_instance + "-" + x.get_name()
            accordion_namespace = ID.generate_k3s_pod_name(application_instance, x.get_name(),
                                                           minicloud)
            port_yaml = []
            if x.get_unit() == 'container':
                host = x.get_host()
                for resource in resource_list:
                    if resource.get_cpu() and resource.get_mem() and resource.get_disk():
                        resource_yaml = {
                            'requests': {'cpu': resource.get_cpu(),
                                         'memory': resource.get_mem(),
                                         'ephemeral-storage': resource.get_disk()}}
                    if not resource.get_cpu() and resource.get_mem() and resource.get_disk():
                        resource_yaml = {
                            'requests': {
                                'memory': resource.get_mem(),
                                'ephemeral-storage': resource.get_disk()}}

                    if host == resource.get_name():
                        filelist = []
                        ports = x.get_port()
                        if ports:
                            i = 0
                            for port in ports:
                                i = i + 1
                                content = {'containerPort': int(port.get('port')), 'name': x.get_name() + str(i)}
                                port_yaml.append(content)
                        if x.get_service():
                            service_port = []
                            if ports:
                                i = 0
                                for port in ports:
                                    i = i + 1
                                    if port.get('protocol') is None:
                                        content = {'name': 'ports' + str(i), 'port': int(port.get('port')),
                                                   'targetPort': int(port.get('port'))}
                                    else:
                                        content = {'name': 'ports' + str(i), 'protocol': port.get('protocol'),
                                                   'port': int(port.get('port')),
                                                   'targetPort': int(port.get('port'))}
                                    service_port.append(content)

                            service_name = accordion_namespace + "-lb"
                            service = {
                                service_name: {
                                    'apiVersion': 'v1',
                                    'kind': 'Service',
                                    'metadata': {
                                        'name': service_name,
                                        'namespace': application_instance,
                                        'labels': {
                                            'app': application_instance}},
                                    'spec': {
                                        'ports': service_port,
                                        'externalIPs': [externalIP],
                                        'selector': {
                                            'type': label},
                                        'type': 'LoadBalancer'}}}
                            if not service_port:
                                service_config = service.get(service_name)
                                spec = service_config.get('spec')
                                spec.pop('ports')
                            service_files.append(service)
                        if x.get_storage_type() == "persistent":
                            persistent_volume = {x.get_volumes_claimname(): {'apiVersion': 'v1',
                                                                             'kind': 'PersistentVolumeClaim',
                                                                             'metadata': {
                                                                                 'name': x.get_volumes_claimname(),
                                                                                 'namespace': application_instance,
                                                                                 'labels': {
                                                                                     'app': application_instance}},
                                                                             'spec':
                                                                                 {'accessModes':
                                                                                      ['ReadWriteOnce'],
                                                                                  'storageClassName': 'local-path',
                                                                                  'resources': {
                                                                                      'requests':
                                                                                          {
                                                                                              'storage': resource.get_disk()}}}}}
                            persistent_files.append(persistent_volume)
                            if port_yaml:
                                deployment = {
                                    accordion_namespace: {'apiVersion': 'apps/v1',
                                                          'kind': 'Deployment',
                                                          'metadata': {
                                                              'name': accordion_namespace,
                                                              'namespace': application_instance,
                                                              'labels': {
                                                                  'app': application_instance,
                                                                  'component': accordion_namespace,
                                                                  'type': label}},
                                                          'spec': {
                                                              'selector': {
                                                                  'matchLabels': {
                                                                      'app': application_instance,
                                                                      'component': accordion_namespace,
                                                                      'type': label}},
                                                              'strategy': {
                                                                  'type': 'Recreate'},
                                                              'template': {
                                                                  'metadata': {
                                                                      'labels': {
                                                                          'app': application_instance,
                                                                          'component': accordion_namespace,
                                                                          'type': label}},
                                                                  'spec': {
                                                                      'containers': [
                                                                          {
                                                                              'image': x.get_image(),
                                                                              'env': [
                                                                                  {
                                                                                      'name': 'ACCORDION_ID',
                                                                                      'value': application_instance}],
                                                                              'name': x.get_name(),
                                                                              'resources': resource_yaml,
                                                                              'imagePullPolicy': 'Always',
                                                                              'ports':
                                                                                  port_yaml,
                                                                              'volumeMounts': [
                                                                                  {
                                                                                      'name': x.get_volumeMounts_name(),
                                                                                      'mountPath': x.get_volumeMounts_path()}]}],
                                                                      'volumes': [{
                                                                          'name': x.get_volumes_name(),
                                                                          'persistentVolumeClaim': {
                                                                              'claimName': x.get_volumes_claimname()}}],
                                                                      'nodeSelector': {
                                                                          'beta.kubernetes.io/os': resource.get_os(),
                                                                          'beta.kubernetes.io/arch': resource.get_arch(),
                                                                          'resource': ''.join(
                                                                              [i for
                                                                               i
                                                                               in
                                                                               resource.get_name()
                                                                               if
                                                                               not i.isdigit()])},
                                                                      'imagePullSecrets': [
                                                                          {
                                                                              'name': application_instance + '-registry-credentials'}]}}}}}
                                deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                          resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                                deployment_files.append(deployment)
                            if not port_yaml:
                                deployment = {
                                    accordion_namespace: {'apiVersion': 'apps/v1',
                                                          'kind': 'Deployment',
                                                          'metadata': {
                                                              'name': accordion_namespace,
                                                              'namespace': application_instance,
                                                              'labels': {
                                                                  'app': application_instance,
                                                                  'component': accordion_namespace, 'type': label}},
                                                          'spec': {
                                                              'selector': {
                                                                  'matchLabels': {
                                                                      'app': application_instance,
                                                                      'component': accordion_namespace, 'type': label}},
                                                              'strategy': {
                                                                  'type': 'Recreate'},
                                                              'template': {
                                                                  'metadata': {
                                                                      'labels': {
                                                                          'app': application_instance,
                                                                          'component': accordion_namespace,
                                                                          'type': label}},
                                                                  'spec': {
                                                                      'containers': [
                                                                          {
                                                                              'image': x.get_image(),
                                                                              'env': [
                                                                                  {
                                                                                      'name': 'ACCORDION_ID',
                                                                                      'value': application_instance}],
                                                                              'name': x.get_name(),
                                                                              'resources': resource_yaml,
                                                                              'imagePullPolicy': 'Always',
                                                                              'volumeMounts': [
                                                                                  {
                                                                                      'name': x.get_volumeMounts_name(),
                                                                                      'mountPath': x.get_volumeMounts_path()}]}],
                                                                      'volumes': [{
                                                                          'name': x.get_volumes_name(),
                                                                          'persistentVolumeClaim': {
                                                                              'claimName': x.get_volumes_claimname()}}],
                                                                      'nodeSelector': {
                                                                          'beta.kubernetes.io/os': resource.get_os(),
                                                                          'beta.kubernetes.io/arch': resource.get_arch(),
                                                                          'resource': ''.join(
                                                                              [i for
                                                                               i
                                                                               in
                                                                               resource.get_name()
                                                                               if
                                                                               not i.isdigit()])},
                                                                      'imagePullSecrets': [
                                                                          {
                                                                              'name': application_instance + '-registry-credentials'}]}}}}}
                                deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                          resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                                deployment_files.append(deployment)
                        if x.get_storage_type() == "ephemeral":
                            if port_yaml:
                                deployment = {
                                    ID.generate_k3s_pod_name(application_instance,
                                                             x.get_name(),
                                                             minicloud)
                                    : {'apiVersion': 'apps/v1',
                                       'kind': 'Deployment',
                                       'metadata': {
                                           'name': ID.generate_k3s_pod_name(application_instance,
                                                                            x.get_name(),
                                                                            minicloud),
                                           'namespace': application_instance,
                                           'labels': {
                                               'app': application_instance,
                                               'component': accordion_namespace, 'type': label}},
                                       'spec': {
                                           'selector': {
                                               'matchLabels': {
                                                   'app': application_instance,
                                                   'component': accordion_namespace, 'type': label}},
                                           'strategy': {
                                               'type': 'Recreate'},
                                           'template': {
                                               'metadata': {
                                                   'labels': {
                                                       'app': application_instance,
                                                       'component': accordion_namespace, 'type': label}},
                                               'spec': {
                                                   'containers': [
                                                       {
                                                           'image': x.get_image(),
                                                           'env': [
                                                               {
                                                                   'name': 'ACCORDION_ID',
                                                                   'value': application_instance}],
                                                           'name': x.get_name(),
                                                           'resources': resource_yaml,
                                                           'imagePullPolicy': 'Always',
                                                           'ports':
                                                               port_yaml,
                                                           'volumeMounts': [
                                                               {
                                                                   'name': "cache-volume",
                                                                   'mountPath': "/cache"}]}],
                                                   'volumes': [{
                                                       'name': "cache-volume",
                                                       'emptyDir': {}}],
                                                   'nodeSelector': {
                                                       'beta.kubernetes.io/os': resource.get_os(),
                                                       'beta.kubernetes.io/arch': resource.get_arch(),
                                                       'resource': ''.join(
                                                           [i for
                                                            i
                                                            in
                                                            resource.get_name()
                                                            if
                                                            not i.isdigit()])},
                                                   'imagePullSecrets': [
                                                       {
                                                           'name': application_instance + '-registry-credentials'}]}}}}}
                                deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                          resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                                deployment_files.append(deployment)
                            if not port_yaml:
                                instances = x.get_instance()
                                i = 0
                                while i < instances:
                                    deployment = {
                                        ID.generate_k3s_pod_name(application_instance,
                                                                 x.get_name() + str(
                                                                     i),
                                                                 minicloud)
                                        : {'apiVersion': 'apps/v1',
                                           'kind': 'Deployment',
                                           'metadata': {
                                               'name': ID.generate_k3s_pod_name(application_instance,
                                                                                x.get_name() + str(
                                                                                    i),
                                                                                minicloud),
                                               'namespace': application_instance,
                                               'labels': {
                                                   'app': application_instance,
                                                   'component': accordion_namespace, 'type': label}},
                                           'spec': {
                                               'selector': {
                                                   'matchLabels': {
                                                       'app': application_instance,
                                                       'component': accordion_namespace, 'type': label}},
                                               'strategy': {
                                                   'type': 'Recreate'},
                                               'template': {
                                                   'metadata': {
                                                       'labels': {
                                                           'app': application_instance,
                                                           'component': accordion_namespace, 'type': label}},
                                                   'spec': {
                                                       'containers': [
                                                           {
                                                               'image': x.get_image(),
                                                               'env': [
                                                                   {
                                                                       'name': 'ACCORDION_ID',
                                                                       'value': application_instance}],
                                                               'name': x.get_name(),
                                                               'resources': resource_yaml,
                                                               'imagePullPolicy': 'Always',
                                                               'volumeMounts': [
                                                                   {
                                                                       'name': "cache-volume",
                                                                       'mountPath': "/cache"}]}],
                                                       'volumes': [{
                                                           'name': "cache-volume",
                                                           'emptyDir': '{}'}],
                                                       'nodeSelector': {
                                                           'beta.kubernetes.io/os': resource.get_os(),
                                                           'beta.kubernetes.io/arch': resource.get_arch(),
                                                           'resource': ''.join(
                                                               [i for
                                                                i
                                                                in
                                                                resource.get_name()
                                                                if
                                                                not i.isdigit()])},
                                                       'imagePullSecrets': [
                                                           {
                                                               'name': application_instance + '-registry-credentials'}]}}}}}
                                    deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                              resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                                    deployment_files.append(deployment)
                                    i = i + 1
                        if x.get_storage_type() == "none":

                            if port_yaml:
                                deployment = {
                                    accordion_namespace: {'apiVersion': 'apps/v1',
                                                          'kind': 'Deployment',
                                                          'metadata': {
                                                              'name': accordion_namespace,
                                                              'namespace': application_instance,
                                                              'labels': {
                                                                  'app': application_instance,
                                                                  'component': accordion_namespace, 'type': label}},
                                                          'spec': {
                                                              'selector': {
                                                                  'matchLabels': {
                                                                      'app': application_instance,
                                                                      'component': accordion_namespace, 'type': label}},
                                                              'strategy': {
                                                                  'type': 'Recreate'},
                                                              'template': {
                                                                  'metadata': {
                                                                      'labels': {
                                                                          'app': application_instance,
                                                                          'component': accordion_namespace,
                                                                          'type': label}},
                                                                  'spec': {
                                                                      'containers': [
                                                                          {
                                                                              'image': x.get_image(),
                                                                              'env': [
                                                                                  {
                                                                                      'name': 'ACCORDION_ID',
                                                                                      'value': application_instance}],
                                                                              'name': x.get_name(),
                                                                              'resources': resource_yaml,
                                                                              'imagePullPolicy': 'Always',
                                                                              'ports':
                                                                                  port_yaml}],
                                                                      'nodeSelector': {
                                                                          'beta.kubernetes.io/os': resource.get_os(),
                                                                          'beta.kubernetes.io/arch': resource.get_arch(),
                                                                          'resource': ''.join(
                                                                              [i for
                                                                               i
                                                                               in
                                                                               resource.get_name()
                                                                               if
                                                                               not i.isdigit()])},
                                                                      'imagePullSecrets': [
                                                                          {
                                                                              'name': application_instance + '-registry-credentials'}]}}}}}
                                deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                          resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                                deployment_files.append(deployment)
                            if not port_yaml:
                                deployment = {
                                    accordion_namespace: {'apiVersion': 'apps/v1',
                                                          'kind': 'Deployment',
                                                          'metadata': {
                                                              'name': accordion_namespace,
                                                              'namespace': application_instance,
                                                              'labels': {
                                                                  'app': application_instance,
                                                                  'component': accordion_namespace, 'type': label}},
                                                          'spec': {
                                                              'selector': {
                                                                  'matchLabels': {
                                                                      'app': application_instance,
                                                                      'component': accordion_namespace, 'type': label}},
                                                              'strategy': {
                                                                  'type': 'Recreate'},
                                                              'template': {
                                                                  'metadata': {
                                                                      'labels': {
                                                                          'app': application_instance,
                                                                          'component': accordion_namespace,
                                                                          'type': label}},
                                                                  'spec': {
                                                                      'containers': [
                                                                          {
                                                                              'image': x.get_image(),
                                                                              'env': [
                                                                                  {
                                                                                      'name': 'ACCORDION_ID',
                                                                                      'value': application_instance}],
                                                                              'name': x.get_name(),
                                                                              'resources': resource_yaml,
                                                                              'imagePullPolicy': 'Always',
                                                                          }],
                                                                      'nodeSelector': {
                                                                          'beta.kubernetes.io/os': resource.get_os(),
                                                                          'beta.kubernetes.io/arch': resource.get_arch(),
                                                                          'resource': ''.join(
                                                                              [i for
                                                                               i
                                                                               in
                                                                               resource.get_name()
                                                                               if
                                                                               not i.isdigit()])},
                                                                      'imagePullSecrets': [
                                                                          {
                                                                              'name': application_instance + '-registry-credentials'}]}}}}}
                                deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                          resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                                deployment_files.append(deployment)

            else:
                host = x.get_host()
                vm_flag = True
                for resource in resource_list:
                    if resource.get_cpu() and resource.get_mem() and resource.get_disk():
                        resource_yaml = {
                            'requests': {'cpu': resource.get_cpu(),
                                         'memory': resource.get_mem(),
                                         'ephemeral-storage': resource.get_disk()}}

                    if host == resource.get_name():
                        if x.get_storage_type() == "persistent":
                            persistent_volume = {x.get_volumes_claimname(): {'apiVersion': 'v1',
                                                                             'kind': 'PersistentVolumeClaim',
                                                                             'metadata': {
                                                                                 'name': 'winhd',
                                                                                 'namespace': application_instance},
                                                                             'spec':
                                                                                 {'accessModes':
                                                                                      ['ReadWriteOnce'],
                                                                                  'storageClassName': 'local-path',
                                                                                  'resources': {
                                                                                      'requests':
                                                                                          {
                                                                                              'storage': resource.get_disk()}}}}}
                            persistent_files.append(persistent_volume)
                        if x.get_service():
                            if x.get_service():
                                service_port_udp = []
                                service_port_tcp = []
                                service_port = []
                                ports = x.get_port()
                                udp_count = 0
                                tcp_count = 0
                                no_protocol_count = 0
                                for port in ports:
                                    if port.get('protocol') == 'UDP':
                                        udp_count = udp_count + 1
                                        content = {'name': 'a' + str(udp_count), 'protocol': port.get('protocol'),
                                                   'port': int(port.get('port')),
                                                   'targetPort': int(port.get('port'))}
                                        service_port_udp.append(content)
                                    if port.get('protocol') == 'TCP':
                                        tcp_count = tcp_count + 1
                                        content = {'name': 'a' + str(tcp_count), 'protocol': port.get('protocol'),
                                                   'port': int(port.get('port')),
                                                   'targetPort': int(port.get('port'))}
                                        service_port_tcp.append(content)
                                    if not port.get('protocol'):
                                        no_protocol_count = no_protocol_count + 1
                                        content = {'name': 'a' + str(no_protocol_count), 'port': int(port.get('port')),
                                                   'targetPort': int(port.get('port'))}
                                        service_port.append(content)
                                if service_port_udp:
                                    service_name = accordion_namespace + "-udp" + "-lb"
                                    service = {
                                        service_name: {
                                            'apiVersion': 'v1',
                                            'kind': 'Service',
                                            'metadata': {
                                                'name': service_name,
                                                'namespace': application_instance,
                                                'labels': {
                                                    'app': application_instance}},
                                            'spec': {
                                                'ports': service_port_udp,
                                                'externalIPs': [externalIP],
                                                'selector': {
                                                    'type': label},
                                                'type': 'LoadBalancer'}}}
                                    service_files.append(service)
                                if service_port_tcp:
                                    service_name = accordion_namespace + "-tcp" + "-lb"
                                    service = {
                                        service_name: {
                                            'apiVersion': 'v1',
                                            'kind': 'Service',
                                            'metadata': {
                                                'name': service_name,
                                                'namespace': application_instance,
                                                'labels': {
                                                    'app': application_instance}},
                                            'spec': {
                                                'ports': service_port_tcp,
                                                'externalIPs': [externalIP],
                                                'selector': {
                                                    'type': label},
                                                'type': 'LoadBalancer'}}}
                                    service_files.append(service)
                                if service_port:
                                    service_name = accordion_namespace + "-lb"
                                    service = {
                                        service_name: {
                                            'apiVersion': 'v1',
                                            'kind': 'Service',
                                            'metadata': {
                                                'name': service_name,
                                                'namespace': application_instance,
                                                'labels': {
                                                    'app': application_instance}},
                                            'spec': {
                                                'ports': service_port,
                                                'externalIPs': [externalIP],
                                                'selector': {
                                                    'type': label},
                                                'type': 'LoadBalancer'}}}
                                    service_files.append(service)
                                if not service_port_udp and not service_port_tcp:
                                    service_name = accordion_namespace + "-lb"
                                    service = {
                                        service_name: {
                                            'apiVersion': 'v1',
                                            'kind': 'Service',
                                            'metadata': {
                                                'name': service_name,
                                                'namespace': application_instance,
                                                'labels': {
                                                    'app': application_instance}},
                                            'spec': {
                                                'ports': service_port_udp,
                                                'externalIPs': [externalIP],
                                                'selector': {
                                                    'type': label},
                                                'type': 'LoadBalancer'}}}
                        for gpu in gpu_list:
                            component = gpu.get('component')
                            if "relayserver" in accordion_namespace:
                                dv = "rs-imp-w2k12-vm"
                            if "localservice" in accordion_namespace:
                                dv = "ls-imp-w2k12-vm"
                            if component not in accordion_namespace:
                                deployment = {
                                    application_instance + "-" + x.get_name() + "-" + minicloud: {
                                        'apiVersion': 'kubevirt.io/v1',
                                        'kind': 'VirtualMachine',
                                        'metadata': {
                                            'name': accordion_namespace,
                                            'namespace': application_instance,
                                            'labels': {
                                                'kubevirt.io/os': 'linux'}},
                                        'spec': {
                                            'running': True,
                                            'template': {
                                                'metadata': {
                                                    'name': accordion_namespace,
                                                    'namespace': application_instance,
                                                    'labels': {
                                                        'app': 'monitorable-vm-windows-exporter',
                                                        'component': accordion_namespace,
                                                        'type': label,
                                                        'kubevirt.io/domain': x.get_flavor()}},
                                                'spec': {'nodeSelector': {
                                                    'beta.kubernetes.io/os': resource.get_os(),
                                                    'beta.kubernetes.io/arch': resource.get_arch()},
                                                    'domain': {'cpu': {
                                                        'cores': int(
                                                            resource.get_cpu())},
                                                        'devices': {
                                                            'interfaces': [{'name': 'default', 'masquerade': {},'ports':ports}],
                                                            'disks': [{
                                                                'disk': {
                                                                    'bus': 'sata'},
                                                                'name': 'disk0'},
                                                                {'cdrom': {
                                                                    'bus': 'sata'},
                                                                    'name': 'virtiocontainerdisk'}

                                                            ]},
                                                        'machine': {
                                                            'type': 'q35'},
                                                        'resources': {
                                                            'requests': {
                                                                'memory': resource.get_mem()}}},
                                                    'volumes': [
                                                        {
                                                            'name': 'disk0',
                                                            'persistentVolumeClaim': {
                                                                'claimName': dv}},
                                                        {
                                                            'name': 'virtiocontainerdisk',
                                                            'containerDisk': {
                                                                'image': 'kubevirt/virtio-container-disk'}},
                                                    ]}}}}}
                                deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                          resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                            else:
                                gpu_model = gpu.get('gpu_model')
                                deployment = {
                                    application_instance + "-" + x.get_name() + "-" + minicloud: {
                                        'apiVersion': 'kubevirt.io/v1',
                                        'kind': 'VirtualMachine',
                                        'metadata': {
                                            'name': accordion_namespace,
                                            'namespace': application_instance,
                                            'labels': {
                                                'kubevirt.io/os': 'linux'}},
                                        'spec': {
                                            'running': True,
                                            'template': {
                                                'metadata': {
                                                    'name': accordion_namespace,
                                                    'namespace': application_instance,
                                                    'labels': {
                                                        'app': 'monitorable-vm-windows-exporter',
                                                        'component': accordion_namespace,
                                                        'type': label,
                                                        'kubevirt.io/domain': x.get_flavor()}},
                                                'spec': {'nodeSelector': {
                                                    'beta.kubernetes.io/os': resource.get_os(),
                                                    'beta.kubernetes.io/arch': resource.get_arch()},
                                                    'domain': {'cpu': {
                                                        'cores': int(
                                                            resource.get_cpu())},
                                                        'devices': {
                                                            'interfaces': [{'name': 'default', 'masquerade': {},
                                                                            'ports': ports}],
                                                            'disks': [{
                                                                'disk': {
                                                                    'bus': 'sata'},
                                                                'name': 'disk0'},
                                                                {'cdrom': {
                                                                    'bus': 'sata'},
                                                                    'name': 'virtiocontainerdisk'}

                                                            ], 'gpus': [
                                                                {'name': 'gpu1', 'deviceName': gpu_model}]},
                                                        'machine': {
                                                            'type': 'q35'},
                                                        'resources': {
                                                            'requests': {
                                                                'memory': resource.get_mem()}}},
                                                    'volumes': [
                                                        {
                                                            'name': 'disk0',
                                                            'persistentVolumeClaim': {
                                                                'claimName': dv}},
                                                        {
                                                            'name': 'virtiocontainerdisk',
                                                            'containerDisk': {
                                                                'image': 'kubevirt/virtio-container-disk'}},
                                                    ]}}}}}

                                deployment = extra_labels(deployment, resource.get_gpu_brand(),
                                                          resource.get_gpu_dedicated(), resource.get_wifi_antenna())
                        deployment_files.append(deployment)
    if vm_flag:
        exporter_service = {
            application_instance + "-" + x.get_name() + "-" + minicloud + "-exporter-service": {
                "apiVersion": "v1",
                "kind": "Service",
                "metadata": {
                    "name": "monitorable-vm-windows-exporter",
                    "namespace": application_instance,
                    "labels": {
                        "app": "monitorable-vm-windows-exporter"
                    }
                },
                "spec": {
                    "ports": [
                        {
                            "name": "metrics",
                            "targetPort": 9182,
                            "port": 9182
                        }
                    ],
                    "selector": {
                        "app": "monitorable-vm-windows-exporter"
                    }
                }
            }}
        service_files.append(exporter_service)
    return deployment_files, persistent_files, service_files


def secret_generation(json, application):
    secret = {application: {'apiVersion': 'v1',
                            'kind': 'Secret',
                            'metadata': {
                                'name': application + '-registry-credentials',
                                'namespace': application},
                            'type': 'kubernetes.io/dockerconfigjson',
                            'data': {
                                '.dockerconfigjson': json}}}
    return secret


def namespace(application):
    namespace = {
        application: {'apiVersion': 'v1', 'kind': 'Namespace',
                      'metadata': {'name': application}}}

    return namespace


def extra_labels(deployment_file, gpu_model, gpu_dedicated, wifi_antennas):
    if gpu_model != "None" and gpu_dedicated != "None":
        key = deployment_file.keys()
        for k in key:
            file = deployment_file[k]
            spec = file['spec']
            template = spec['template']
            template_spec = template['spec']
            nodeselector = template_spec['nodeSelector']

            nodeselector['GPU.model'] = gpu_model
            nodeselector['GPU.dedicated'] = str(gpu_dedicated).lower()

    if wifi_antennas != "None":
        key = deployment_file.keys()
        for k in key:
            file = deployment_file[k]
            spec = file['spec']
            template = spec['template']
            template_spec = template['spec']
            nodeselector = template_spec['nodeSelector']
            nodeselector['Wifi.External.Antenna'] = str(wifi_antennas).lower()
    return deployment_file
